@@ -21,7 +21,7 @@ import enum
 import math
 import datetime as dt
 import operator
-from decimal import Context, Decimal
+from decimal import Context, Decimal, localcontext
 from inspect import isclass, isfunction
 from fractions import Fraction
 from functools import reduce
@@ -33,14 +33,17 @@ from hypothesis.internal.cache import LRUReusedCache
 from hypothesis.searchstrategy import SearchStrategy
 from hypothesis.internal.compat import gcd, ceil, floor, hrange, \
     text_type, get_type_hints, getfullargspec, implements_iterator
-from hypothesis.internal.floats import is_negative, float_to_int, \
-    int_to_float, count_between_floats
+from hypothesis.internal.floats import next_up, next_down, is_negative, \
+    float_to_int, int_to_float, count_between_floats
+from hypothesis.internal.charmap import as_general_categories
+from hypothesis.internal.cathetus import cathetus
 from hypothesis.internal.renaming import renamed_arguments
 from hypothesis.utils.conventions import infer, not_set
 from hypothesis.internal.reflection import proxies, required_args
 from hypothesis.internal.validation import check_type, try_convert, \
-    check_strategy, check_valid_bound, check_valid_sizes, \
-    check_valid_integer, check_valid_interval
+    check_strategy, check_valid_size, check_valid_bound, \
+    check_valid_sizes, check_valid_integer, check_valid_interval, \
+    check_valid_magnitude
 
 __all__ = [
     'nothing',
@@ -122,7 +125,6 @@ def base_defines_strategy(force_reusable):
     If force_reusable is True, the generated values are assumed to be
     reusable, i.e. immutable and safe to cache, across multiple test
     invocations.
-
     """
     def decorator(strategy_definition):
         """A decorator that registers the function as a strategy and makes it
@@ -179,7 +181,6 @@ def nothing():
     an attempt to draw.
 
     Examples from this strategy do not shrink (because there are none).
-
     """
     return NOTHING
 
@@ -194,7 +195,6 @@ def just(value):
     of ``just(callable())`` to get a fresh value each time.
 
     Examples from this strategy do not shrink (because there is only one).
-
     """
     from hypothesis.searchstrategy.misc import JustStrategy
 
@@ -207,7 +207,6 @@ def none():
 
     Examples from this strategy do not shrink (because there is only
     one).
-
     """
     return just(None)
 
@@ -217,7 +216,7 @@ def one_of(*args):
     strategies.
 
     This may be called with one iterable argument instead of multiple
-    strategy arguments. In which case one_of(x) and one_of(\*x) are
+    strategy arguments. In which case ``one_of(x)`` and ``one_of(*x)`` are
     equivalent.
 
     Examples from this strategy will generally shrink to ones that come from
@@ -230,7 +229,6 @@ def one_of(*args):
     ``x = st.deferred(lambda: st.none() | st.tuples(x, x))`` will shrink well,
     but ``x = st.deferred(lambda: st.tuples(x, x) | st.none())`` will shrink
     very badly indeed.
-
     """
     if len(args) == 1 and not isinstance(args[0], SearchStrategy):
         try:
@@ -252,7 +250,6 @@ def integers(min_value=None, max_value=None):
 
     Examples from this strategy will shrink towards zero, and negative values
     will also shrink towards positive (i.e. -n may be replaced by +n).
-
     """
 
     check_valid_bound(min_value, 'min_value')
@@ -302,7 +299,6 @@ def booleans():
 
     Examples from this strategy will shrink towards False (i.e.
     shrinking will try to replace True with False where possible).
-
     """
     from hypothesis.searchstrategy.misc import BoolStrategy
     return BoolStrategy()
@@ -329,7 +325,6 @@ def floats(
     shrinking behaviour, but it tries to improve "human readability". Finite
     numbers will be preferred to infinity and infinity will be preferred to
     NaN.
-
     """
 
     if allow_nan is None:
@@ -341,17 +336,32 @@ def floats(
                     allow_nan
                 ))
 
-    min_value = try_convert(float, min_value, 'min_value')
-    max_value = try_convert(float, max_value, 'max_value')
-
     check_valid_bound(min_value, 'min_value')
     check_valid_bound(max_value, 'max_value')
+
+    min_arg, max_arg = min_value, max_value
+    if min_value is not None:
+        min_value = float(min_value)
+    if max_value is not None:
+        max_value = float(max_value)
+
     check_valid_interval(min_value, max_value, 'min_value', 'max_value')
     if min_value == float(u'-inf'):
         min_value = None
     if max_value == float(u'inf'):
         max_value = None
 
+    if min_value is not None and min_value < min_arg:
+        min_value = next_up(min_value)
+        assert min_value > min_arg
+    if max_value is not None and max_value > max_arg:
+        max_value = next_down(max_value)
+        assert max_value < max_arg
+    if None not in (min_value, max_value) and min_value > max_value:
+        raise InvalidArgument(
+            'There are no floating-point values between min_value=%r and '
+            'max_value=%r' % (min_arg, max_arg))
+
     if allow_infinity is None:
         allow_infinity = bool(min_value is None or max_value is None)
     elif allow_infinity:
@@ -422,21 +432,6 @@ def floats(
         return result
 
 
-@cacheable
-@defines_strategy_with_reusable_values
-def complex_numbers():
-    """Returns a strategy that generates complex numbers.
-
-    Examples from this strategy shrink by shrinking their component real
-    and imaginary parts.
-
-    """
-    from hypothesis.searchstrategy.numbers import ComplexStrategy
-    return ComplexStrategy(
-        tuples(floats(), floats())
-    )
-
-
 @cacheable
 @defines_strategy
 def tuples(*args):
@@ -447,13 +442,12 @@ def tuples(*args):
     two with both values an integer.
 
     Examples from this strategy shrink by shrinking their component parts.
-
     """
     for arg in args:
         check_strategy(arg)
 
     from hypothesis.searchstrategy.collections import TupleStrategy
-    return TupleStrategy(args, tuple)
+    return TupleStrategy(args)
 
 
 @defines_strategy
@@ -471,7 +465,6 @@ def sampled_from(elements):
     the list. So e.g. sampled_from((10, 1)) will shrink by trying to replace
     1 values with 10, and sampled_from((1, 10)) will shrink by trying to
     replace 10 values with 1.
-
     """
     from hypothesis.searchstrategy.misc import SampledFromStrategy
     from hypothesis.internal.conjecture.utils import check_sample
@@ -489,9 +482,6 @@ def sampled_from(elements):
     return SampledFromStrategy(values)
 
 
-_AVERAGE_LIST_LENGTH = 5.0
-
-
 @cacheable
 @defines_strategy
 def lists(
@@ -503,9 +493,8 @@ def lists(
     None). If max_size is 0 then elements may be None and only the empty list
     will be drawn.
 
-    average_size may be used as a size hint to roughly control the size
-    of the list but it may not be the actual average of sizes you get, due
-    to a variety of factors.
+    The average_size argument is deprecated.  Internal upgrades since
+    Hypothesis 1.x mean we no longer needed this hint to generate useful data.
 
     If unique is True (or something that evaluates to True), we compare direct
     object equality, as if unique_by was `lambda x: x`. This comparison only
@@ -517,16 +506,25 @@ def lists(
 
     Examples from this strategy shrink by trying to remove elements from the
     list, and by shrinking each individual element of the list.
-
     """
     check_valid_sizes(min_size, average_size, max_size)
-    if elements is None or (max_size is not None and max_size <= 0):
-        if max_size is None or max_size > 0:
+    if elements is None:
+        note_deprecation(
+            'Passing a strategy for `elements` of the list will be required '
+            'in a future version of Hypothesis.  To create lists that are '
+            'always empty, use `builds(list)` or `lists(nothing())`.'
+        )
+        if min_size or average_size or max_size:
+            # Checked internally for lists with an elements strategy, but
+            # we're about to skip that and return builds(list) instead...
             raise InvalidArgument(
-                u'Cannot create non-empty lists without an element type'
+                'Cannot create a non-empty collection (min_size=%r, '
+                'average_size=%r, max_size=%r) without elements.'
+                % (min_size, average_size, max_size)
             )
-        else:
-            return builds(list)
+        return builds(list)
+    if max_size == 0:
+        return builds(list)
     check_strategy(elements)
     if unique:
         if unique_by is not None:
@@ -544,29 +542,16 @@ def lists(
     if max_size is None:
         max_size = float('inf')
 
-    if average_size is None:
-        average_size = max(
-            _AVERAGE_LIST_LENGTH,
-            min_size * 2
-        )
-        if max_size < float('inf'):
-            average_size = min(average_size, 0.5 * (min_size + max_size))
-
     from hypothesis.searchstrategy.collections import ListStrategy, \
         UniqueListStrategy
     if unique_by is not None:
         return UniqueListStrategy(
             elements=elements,
-            average_size=average_size,
             max_size=max_size,
             min_size=min_size,
             key=unique_by
         )
-    else:
-        return ListStrategy(
-            (elements,), average_length=average_size,
-            min_size=min_size, max_size=max_size,
-        )
+    return ListStrategy(elements, min_size=min_size, max_size=max_size)
 
 
 @cacheable
@@ -580,8 +565,13 @@ def sets(elements=None, min_size=None, average_size=None, max_size=None):
 
     Examples from this strategy shrink by trying to remove elements from the
     set, and by shrinking each individual element of the set.
-
     """
+    if elements is None:
+        note_deprecation(
+            'Passing a strategy for `elements` of the set will be required '
+            'in a future version of Hypothesis.  To create sets that are '
+            'always empty, use `builds(set)` or `sets(nothing())`.'
+        )
     return lists(
         elements=elements, min_size=min_size, average_size=average_size,
         max_size=max_size, unique=True
@@ -593,6 +583,13 @@ def sets(elements=None, min_size=None, average_size=None, max_size=None):
 def frozensets(elements=None, min_size=None, average_size=None, max_size=None):
     """This is identical to the sets function but instead returns
     frozensets."""
+    if elements is None:
+        note_deprecation(
+            'Passing a strategy for `elements` of the frozenset will be '
+            'required in a future version of Hypothesis.  To create '
+            'frozensets that are always empty, use `builds(frozenset)` '
+            'or `frozensets(nothing())`.'
+        )
     return lists(
         elements=elements, min_size=min_size, average_size=average_size,
         max_size=max_size, unique=True
@@ -608,8 +605,14 @@ def iterables(elements=None, min_size=None, average_size=None, max_size=None,
     fixed length (e.g. generators). This strategy produces iterators,
     which cannot be indexed and do not have a fixed length. This ensures
     that you do not accidentally depend on sequence behaviour.
-
     """
+    if elements is None:
+        note_deprecation(
+            'Passing a strategy for `elements` of the iterable will be '
+            'required in a future version of Hypothesis.  To create '
+            'iterables that are always empty, use `iterables(nothing())`.'
+        )
+
     @implements_iterator
     class PrettyIter(object):
         def __init__(self, values):
@@ -643,7 +646,6 @@ def fixed_dictionaries(mapping):
 
     Examples from this strategy shrink by shrinking each individual value in
     the generated dictionary.
-
     """
     from hypothesis.searchstrategy.collections import FixedKeysDictStrategy
     check_type(dict, mapping, 'mapping')
@@ -665,7 +667,6 @@ def dictionaries(
 
     Examples from this strategy shrink by trying to remove keys from the
     generated dictionary, and by shrinking each generated key and value.
-
     """
     check_valid_sizes(min_size, average_size, max_size)
     if max_size == 0:
@@ -675,7 +676,7 @@ def dictionaries(
 
     return lists(
         tuples(keys, values),
-        min_size=min_size, average_size=average_size, max_size=max_size,
+        min_size=min_size, max_size=max_size,
         unique_by=lambda x: x[0]
     ).map(dict_class)
 
@@ -692,7 +693,6 @@ def streaming(elements):
 
     .. deprecated:: 3.15.0
         Use :func:`data() <hypothesis.strategies.data>` instead.
-
     """
     note_deprecation(
         'streaming() has been deprecated. Use the data() strategy instead and '
@@ -713,39 +713,43 @@ def characters(whitelist_categories=None, blacklist_categories=None,
     """Generates unicode text type (unicode on python 2, str on python 3)
     characters following specified filtering rules.
 
-    When no filtering rules are specifed, any character can be produced.
+    - When no filtering rules are specifed, any character can be produced.
+    - If ``min_codepoint`` or ``max_codepoint`` is specifed, then only
+      characters having a codepoint in that range will be produced.
+    - If ``whitelist_categories`` is specified, then only characters from those
+      Unicode categories will be produced. This is a further restriction,
+      characters must also satisfy ``min_codepoint`` and ``max_codepoint``.
+    - If ``blacklist_categories`` is specified, then any character from those
+      categories will not be produced.  Any overlap between
+      ``whitelist_categories`` and ``blacklist_categories`` will raise an
+      exception, as each character can only belong to a single class.
+    - If ``whitelist_characters`` is specified, then any additional characters
+      in that list will also be produced.
+    - If ``blacklist_characters`` is specified, then any characters in
+      that list will be not be produced. Any overlap between \
+      ``whitelist_characters`` and ``blacklist_characters`` will raise an
+      exception.
 
-    If ``min_codepoint`` or ``max_codepoint`` is specifed, then only
-    characters having a codepoint in that range will be produced.
+    The ``_codepoint`` arguments must be integers between zero and
+    :obj:`python:sys.max_unicode`.  The ``_characters`` arguments must be
+    collections of length-one unicode strings, such as a unicode string.
 
-    If ``whitelist_categories`` is specified, then only characters from those
-    Unicode categories will be produced. This is a further restriction,
-    characters must also satisfy ``min_codepoint`` and ``max_codepoint``.
+    The ``_categories`` arguments must be used to specify either the
+    one-letter Unicode major category or the two-letter Unicode
+    `general category`_.  For example, ``('Nd', 'Lu')`` signifies "Number,
+    decimal digit" and "Letter, uppercase".  A single letter ('major category')
+    can be given to match all corresponding categories, for example ``'P'``
+    for characters in any punctuation category.
 
-    If ``blacklist_categories`` is specified, then any character from those
-    categories will not be produced. This is a further restriction,
-    characters that match both ``whitelist_categories`` and
-    ``blacklist_categories`` will not be produced.
-
-    If ``whitelist_characters`` is specified, then any additional characters
-    in that list will also be produced.
-
-    If ``blacklist_characters`` is specified, then any characters in that list
-    will be not be produced. Any overlap between ``whitelist_characters`` and
-    ``blacklist_characters`` will raise an exception.
-
-    Examples from this strategy shrink towards smaller codepoints.
+    .. _general category: https://wikipedia.org/wiki/Unicode_character_property
 
+    Examples from this strategy shrink towards the codepoint for ``'0'``,
+    or the first allowable codepoint after it if ``'0'`` is excluded.
     """
-    if (
-        min_codepoint is not None and max_codepoint is not None and
-        min_codepoint > max_codepoint
-    ):
-        raise InvalidArgument(
-            'Cannot have min_codepoint=%d > max_codepoint=%d ' % (
-                min_codepoint, max_codepoint
-            )
-        )
+    check_valid_size(min_codepoint, 'min_codepoint')
+    check_valid_size(max_codepoint, 'max_codepoint')
+    check_valid_interval(min_codepoint, max_codepoint,
+                         'min_codepoint', 'max_codepoint')
     if all((whitelist_characters is not None,
             min_codepoint is None,
             max_codepoint is None,
@@ -753,25 +757,33 @@ def characters(whitelist_categories=None, blacklist_categories=None,
             blacklist_categories is None,
             )):
         raise InvalidArgument(
-            'Cannot have just whitelist_characters=%r alone, '
-            'it would have no effect. Perhaps you want sampled_from()' % (
-                whitelist_characters,
-            )
-        )
-    if (
-        whitelist_characters is not None and
-        blacklist_characters is not None and
-        set(blacklist_characters).intersection(set(whitelist_characters))
-    ):
+            'Passing only whitelist_characters=%r would have no effect. '
+            'Perhaps you want sampled_from() ?' % (whitelist_characters,))
+    blacklist_characters = blacklist_characters or ''
+    whitelist_characters = whitelist_characters or ''
+    overlap = set(blacklist_characters).intersection(whitelist_characters)
+    if overlap:
         raise InvalidArgument(
             'Characters %r are present in both whitelist_characters=%r, and '
             'blacklist_characters=%r' % (
-                set(blacklist_characters).intersection(
-                    set(whitelist_characters)
-                ),
-                whitelist_characters, blacklist_characters,
-            )
-        )
+                sorted(overlap), whitelist_characters, blacklist_characters))
+    blacklist_categories = as_general_categories(
+        blacklist_categories, 'blacklist_categories')
+    if whitelist_categories is not None and not whitelist_categories and \
+            not whitelist_characters:
+        raise InvalidArgument(
+            'When whitelist_categories is an empty collection and there are '
+            'no characters specified in whitelist_characters, nothing can '
+            'be generated by the characters() strategy.')
+    whitelist_categories = as_general_categories(
+        whitelist_categories, 'whitelist_categories')
+    both_cats = set(
+        blacklist_categories or ()).intersection(whitelist_categories or ())
+    if both_cats:
+        raise InvalidArgument(
+            'Categories %r are present in both whitelist_categories=%r, and '
+            'blacklist_categories=%r' % (
+                sorted(both_cats), whitelist_categories, blacklist_categories))
 
     from hypothesis.searchstrategy.strings import OneCharStringStrategy
     return OneCharStringStrategy(whitelist_categories=whitelist_categories,
@@ -795,13 +807,16 @@ def text(
     characters). If it is an empty collection this will only generate empty
     strings.
 
-    min_size, max_size and average_size have the usual interpretations.
+    min_size and max_size have the usual interpretations.
+
+    The average_size argument is deprecated.  Internal upgrades since
+    Hypothesis 1.x mean we no longer needed this hint to generate useful data.
 
     Examples from this strategy shrink towards shorter strings, and with the
     characters in the text shrinking as per the alphabet strategy.
-
     """
     from hypothesis.searchstrategy.strings import StringStrategy
+    check_valid_sizes(min_size, average_size, max_size)
     if alphabet is None:
         char_strategy = characters(blacklist_categories=('Cs',))
     elif not alphabet:
@@ -817,8 +832,7 @@ def text(
     else:
         char_strategy = sampled_from(list(map(text_type, alphabet)))
     return StringStrategy(lists(
-        char_strategy, average_size=average_size, min_size=min_size,
-        max_size=max_size
+        char_strategy, min_size=min_size, max_size=max_size
     ))
 
 
@@ -849,7 +863,6 @@ def from_regex(regex):
 
     Examples from this strategy shrink towards shorter strings and lower
     character values.
-
     """
     from hypothesis.searchstrategy.regex import regex_strategy
     return regex_strategy(regex)
@@ -863,11 +876,13 @@ def binary(
     """Generates the appropriate binary type (str in python 2, bytes in python
     3).
 
-    min_size, average_size and max_size have the usual interpretations.
+    min_size and max_size have the usual interpretations.
+
+    The average_size argument is deprecated.  Internal upgrades since
+    Hypothesis 1.x mean we no longer needed this hint to generate useful data.
 
     Examples from this strategy shrink towards smaller strings and lower byte
     values.
-
     """
     from hypothesis.searchstrategy.strings import BinaryStringStrategy, \
         FixedSizeBytes
@@ -877,7 +892,7 @@ def binary(
     return BinaryStringStrategy(
         lists(
             integers(min_value=0, max_value=255),
-            average_size=average_size, min_size=min_size, max_size=max_size
+            min_size=min_size, max_size=max_size
         )
     )
 
@@ -889,7 +904,6 @@ def randoms():
     RandomWithSeed class which displays what it was initially seeded with)
 
     Examples from this strategy shrink to seeds closer to zero.
-
     """
     from hypothesis.searchstrategy.misc import RandomStrategy
     return RandomStrategy(integers())
@@ -918,7 +932,6 @@ def random_module():
     which will give you one.
 
     Examples from these strategy shrink to seeds closer to zero.
-
     """
     from hypothesis.control import cleanup
     import random
@@ -953,7 +966,6 @@ def builds(*callable_and_args, **kwargs):
 
     Examples from this strategy shrink by shrinking the argument values to
     the callable.
-
     """
     if callable_and_args:
         target, args = callable_and_args[0], callable_and_args[1:]
@@ -1026,7 +1038,6 @@ def from_type(thing):
        other elements in the lookup.
     4. Finally, if ``thing`` has type annotations for all required arguments,
        it is resolved via :func:`~hypothesis.strategies.builds`.
-
     """
     from hypothesis.searchstrategy import types
     try:
@@ -1134,7 +1145,6 @@ def fractions(min_value=None, max_value=None, max_denominator=None):
 
     Examples from this strategy shrink towards smaller denominators, then
     closer to zero.
-
     """
     min_value = try_convert(Fraction, min_value, 'min_value')
     max_value = try_convert(Fraction, max_value, 'max_value')
@@ -1217,6 +1227,25 @@ def fractions(min_value=None, max_value=None, max_denominator=None):
         lambda f: f.limit_denominator(max_denominator))
 
 
+def _as_finite_decimal(value, name, allow_infinity):
+    """Convert decimal bounds to decimals, carefully."""
+    assert name in ('min_value', 'max_value')
+    if value is None:
+        return None
+    if not isinstance(value, Decimal):
+        with localcontext(Context()):  # ensure that default traps are enabled
+            value = try_convert(Decimal, value, name)
+    if value.is_finite():
+        return value
+    if value.is_infinite() and (value < 0 if 'min' in name else value > 0):
+        if allow_infinity or allow_infinity is None:
+            return None
+        raise InvalidArgument('allow_infinity=%r, but %s=%r'
+                              % (allow_infinity, name, value))
+    # This could be infinity, quiet NaN, or signalling NaN
+    raise InvalidArgument(u'Invalid %s=%r' % (name, value))
+
+
 @cacheable
 @defines_strategy_with_reusable_values
 def decimals(min_value=None, max_value=None,
@@ -1240,34 +1269,14 @@ def decimals(min_value=None, max_value=None,
 
     Examples from this strategy do not have a well defined shrink order but
     try to maximize human readability when shrinking.
-
     """
     # Convert min_value and max_value to Decimal values, and validate args
     check_valid_integer(places)
     if places is not None and places < 0:
         raise InvalidArgument('places=%r may not be negative' % places)
-
-    if min_value is not None:
-        min_value = try_convert(Decimal, min_value, 'min_value')
-        if min_value.is_infinite() and min_value < 0:
-            if not (allow_infinity or allow_infinity is None):
-                raise InvalidArgument('allow_infinity=%r, but min_value=%r'
-                                      % (allow_infinity, min_value))
-            min_value = None
-        elif not min_value.is_finite():
-            # This could be positive infinity, quiet NaN, or signalling NaN
-            raise InvalidArgument(u'Invalid min_value=%r' % min_value)
-    if max_value is not None:
-        max_value = try_convert(Decimal, max_value, 'max_value')
-        if max_value.is_infinite() and max_value > 0:
-            if not (allow_infinity or allow_infinity is None):
-                raise InvalidArgument('allow_infinity=%r, but max_value=%r'
-                                      % (allow_infinity, max_value))
-            max_value = None
-        elif not max_value.is_finite():
-            raise InvalidArgument(u'Invalid max_value=%r' % max_value)
+    min_value = _as_finite_decimal(min_value, 'min_value', allow_infinity)
+    max_value = _as_finite_decimal(max_value, 'max_value', allow_infinity)
     check_valid_interval(min_value, max_value, 'min_value', 'max_value')
-
     if allow_infinity and (None not in (min_value, max_value)):
         raise InvalidArgument('Cannot allow infinity between finite bounds')
     # Set up a strategy for finite decimals.  Note that both floating and
@@ -1350,7 +1359,6 @@ def permutations(values):
 
     Examples from this strategy shrink by trying to become closer to the
     original order of values.
-
     """
     from hypothesis.internal.conjecture.utils import integer_range
 
@@ -1408,7 +1416,6 @@ def datetimes(
     are at the beginning of the list for proper minimisation.
 
     Examples from this strategy shrink towards midnight on January 1st 2000.
-
     """
     # Why must bounds be naive?  In principle, we could also write a strategy
     # that took aware bounds, but the API and validation is much harder.
@@ -1450,7 +1457,6 @@ def dates(
     """A strategy for dates between ``min_date`` and ``max_date``.
 
     Examples from this strategy shrink towards January 1st 2000.
-
     """
     from hypothesis.searchstrategy.datetime import DateStrategy
 
@@ -1477,7 +1483,6 @@ def times(
 
     Examples from this strategy shrink towards midnight, with the timezone
     component shrinking as for the strategy that provided it.
-
     """
     check_type(dt.time, min_value, 'min_value')
     check_type(dt.time, max_value, 'max_value')
@@ -1504,7 +1509,6 @@ def timedeltas(
     """A strategy for timedeltas between ``min_value`` and ``max_value``.
 
     Examples from this strategy shrink towards zero.
-
     """
     from hypothesis.searchstrategy.datetime import TimedeltaStrategy
 
@@ -1527,11 +1531,10 @@ def composite(f):
 
     Examples from this strategy shrink by shrinking the output of each draw
     call.
-
     """
 
     from hypothesis.internal.reflection import define_function_signature
-    from hypothesis.searchstrategy.strategies import calc_label
+    from hypothesis.internal.conjecture.utils import calc_label_from_cls
     argspec = getfullargspec(f)
 
     if (
@@ -1550,7 +1553,7 @@ def composite(f):
               if k in (argspec.args + argspec.kwonlyargs + ['return'])}
     new_argspec = argspec._replace(args=argspec.args[1:], annotations=annots)
 
-    label = calc_label(f)
+    label = calc_label_from_cls(f)
 
     @defines_strategy
     @define_function_signature(f.__name__, f.__doc__, new_argspec)
@@ -1574,6 +1577,86 @@ def composite(f):
     return accept
 
 
+@defines_strategy_with_reusable_values
+@cacheable
+def complex_numbers(min_magnitude=0, max_magnitude=None,
+                    allow_infinity=None, allow_nan=None):
+    """Returns a strategy that generates complex numbers.
+
+    This strategy draws complex numbers with constrained magnitudes.
+    The ``min_magnitude`` and ``max_magnitude`` parameters should be
+    non-negative :class:`~python:numbers.Real` numbers; values
+    of ``None`` correspond to zero and infinite values respectively.
+
+    If ``min_magnitude`` is positive or ``max_magnitude`` is finite, it
+    is an error to enable ``allow_nan``.  If ``max_magnitude`` is finite,
+    it is an error to enable ``allow_infinity``.
+
+    Examples from this strategy shrink by shrinking their real and
+    imaginary parts, as :func:`~hypothesis.strategies.floats`.
+
+    If you need to generate complex numbers with particular real and
+    imaginary parts or relationships between parts, consider using
+    `builds(complex, ...) <hypothesis.strategies.builds>` or
+    `@composite <hypothesis.strategies.composite>` respectively.
+    """
+    check_valid_magnitude(min_magnitude, 'min_magnitude')
+    check_valid_magnitude(max_magnitude, 'max_magnitude')
+    check_valid_interval(min_magnitude, max_magnitude,
+                         'min_magnitude', 'max_magnitude')
+    if max_magnitude == float('inf'):
+        max_magnitude = None
+    if min_magnitude == 0:
+        min_magnitude = None
+
+    if allow_infinity is None:
+        allow_infinity = bool(max_magnitude is None)
+    elif allow_infinity and max_magnitude is not None:
+        raise InvalidArgument(
+            'Cannot have allow_infinity=%r with max_magnitude=%r' %
+            (allow_infinity, max_magnitude)
+        )
+    if allow_nan is None:
+        allow_nan = bool(min_magnitude is None and max_magnitude is None)
+    elif allow_nan and not (min_magnitude is None and max_magnitude is None):
+        raise InvalidArgument(
+            'Cannot have allow_nan=%r, min_magnitude=%r max_magnitude=%r' %
+            (allow_nan, min_magnitude, max_magnitude)
+        )
+    allow_kw = dict(allow_nan=allow_nan, allow_infinity=allow_infinity)
+
+    if min_magnitude is None and max_magnitude is None:
+        # In this simple but common case, there are no constraints on the
+        # magnitude and therefore no relationship between the real and
+        # imaginary parts.
+        return builds(complex, floats(**allow_kw), floats(**allow_kw))
+
+    @composite
+    def constrained_complex(draw):
+        # Draw the imaginary part, and determine the maximum real part given
+        # this and the max_magnitude
+        if max_magnitude is None:
+            zi = draw(floats(**allow_kw))
+            rmax = float('inf')
+        else:
+            zi = draw(floats(-max_magnitude, max_magnitude, **allow_kw))
+            rmax = cathetus(max_magnitude, zi)
+        # Draw the real part from the allowed range given the imaginary part
+        if min_magnitude is None or math.fabs(zi) >= min_magnitude:
+            zr = draw(floats(-rmax, rmax, **allow_kw))
+        else:
+            zr = draw(floats(cathetus(min_magnitude, zi), rmax, **allow_kw))
+        # Order of conditions carefully tuned so that for a given pair of
+        # magnitude arguments, we always either draw or do not draw the bool
+        # (crucial for good shrinking behaviour) but only invert when needed.
+        if min_magnitude is not None and draw(booleans()) and \
+                math.fabs(zi) <= min_magnitude:
+            zr = -zr
+        return complex(zr, zi)
+
+    return constrained_complex()
+
+
 def shared(base, key=None):
     """Returns a strategy that draws a single shared value per run, drawn from
     base. Any two shared instances with the same key will share the same value,
@@ -1590,7 +1673,6 @@ def shared(base, key=None):
     >>> y = shared(s, key="hi")
 
     Examples from this strategy shrink as per their base strategy.
-
     """
     from hypothesis.searchstrategy.shared import SharedStrategy
     return SharedStrategy(base, key)
@@ -1609,7 +1691,6 @@ def choices():
 
     Examples from this strategy shrink by making each choice function return
     an earlier value in the sequence passed to it.
-
     """
     from hypothesis.control import note, current_build_context
     from hypothesis.internal.conjecture.utils import choice, check_sample
@@ -1664,7 +1745,6 @@ def uuids(version=None):
     ``lists(uuids())`` the resulting list will never contain duplicates.
 
     Examples from this strategy don't have any meaningful shrink order.
-
     """
     from uuid import UUID
     if version not in (None, 1, 2, 3, 4, 5):
@@ -1687,7 +1767,6 @@ def runner(default=not_set):
     that default. If no default is provided, raises InvalidArgument.
 
     Examples from this strategy do not shrink (because there is only one).
-
     """
     class RunnerStrategy(SearchStrategy):
 
@@ -1720,7 +1799,6 @@ def data():
 
     Examples from this strategy do not shrink (because there is only one),
     but the result of calls to each draw() call shrink as they normally would.
-
     """
     from hypothesis.control import note
 
@@ -1789,7 +1867,6 @@ def register_type_strategy(custom_type, strategy):
 
     ``strategy`` may be a search strategy, or a function that takes a type and
     returns a strategy (useful for generic types).
-
     """
     from hypothesis.searchstrategy import types
     if not isinstance(custom_type, type):
@@ -1834,7 +1911,6 @@ def deferred(definition):
 
     Examples from this strategy shrink as they normally would from the strategy
     returned by the definition.
-
     """
     from hypothesis.searchstrategy.deferred import DeferredStrategy
     return DeferredStrategy(definition)
